<?php

namespace LWS\WOOREWARDS\Coupons\Events;

// don't call the file directly
if (!defined('ABSPATH')) exit();

/** Earn points for using a coupon in an order.
 *	Coupon can be filtered by code with a pattern (support wildcards). */
class UseCoupon extends \LWS\WOOREWARDS\Abstracts\Event
implements \LWS\WOOREWARDS\PRO\Events\I_CartPreview
{
	use \LWS\WOOREWARDS\PRO\Events\T_Order;
	use \LWS\WOOREWARDS\PRO\Events\T_SponsorshipOrigin;

	function getInformation()
	{
		return array_merge(parent::getInformation(), array(
			'icon'  => 'lws-icon-coupon',
			'short' => __("Earn points for using a coupon", 'woorewards-points-for-coupon'),
			'help'  => __("Points are given after the order is completed", 'woorewards-points-for-coupon'),
		));
	}

	function getDescription($context = 'backend')
	{
		return sprintf(__("Use the coupon %s on an order", 'woorewards-points-for-coupon'), "<strong>" . $this->getCouponCode() . "</strong>");
	}

	public function getDisplayType()
	{
		return _x("Use a Coupon", "getDisplayType", 'woorewards-points-for-coupon');
	}

	/**	Event categories, used to filter out events from pool options.
	 *	@return array with category_id => category_label. */
	public function getCategories()
	{
		return array_merge(parent::getCategories(), array(
			'woocommerce' => __("WooCommerce", 'woorewards-points-for-coupon'),
			'order'  => __("Order", 'woorewards-points-for-coupon')
		));
	}

	function getData()
	{
		$prefix = $this->getDataKeyPrefix();
		$data = parent::getData();
		$data[$prefix . 'coupon_code'] = $this->getCouponCode();
		return $data;
	}

	function getForm($context = 'editlist')
	{
		$prefix = $this->getDataKeyPrefix($context = 'editlist');
		$form = parent::getForm($context);
		$form .= $this->getFieldsetBegin(2, __("Options", 'woorewards-points-for-coupon'));

		// Minimum order amount
		$label = __("Coupon Code", 'woorewards-points-for-coupon');
		$tooltip = implode('<br/>', array(
			sprintf(__("Accept %s to mach several codes.", 'woorewards-points-for-coupon'), sprintf(
				'<a target="_blank" href="%s">%s</a>',
				\esc_attr('https://www.computerhope.com/jargon/w/wildcard.htm'),
				__("Wildcard characters")
			)),
			__('The two wildcard characters are * and ?, and both can come in the middle or end of a word. The escaped question mark stands for one character and the star stands for any number of characters.', 'woorewards-points-for-coupon'),
		));
		$form .= "<div class='field-help'>$tooltip</div>";
		$form .= "<div class='lws-$context-opt-title label'>$label<div class='bt-field-help'>?</div></div>";
		$form .= "<div class='lws-$context-opt-input value'><input type='text' id='{$prefix}coupon_code' name='{$prefix}coupon_code' placeholder='ABC125'/></div>";

		$form .= $this->getFieldsetEnd(2);
		return $form;
	}

	function submit($form = array(), $source = 'editlist')
	{
		$prefix = $this->getDataKeyPrefix();
		$values = \apply_filters('lws_adminpanel_arg_parse', array(
			'post'     => ($source == 'post'),
			'values'   => $form,
			'format'   => array(
				$prefix . 'coupon_code' => 'S',
			),
			'defaults' => array(
				$prefix . 'coupon_code' => '',
			),
			'labels'   => array(
				$prefix . 'coupon_code' => __("Coupon Code", 'woorewards-points-for-coupon'),
			)
		));
		if (!(isset($values['valid']) && $values['valid']))
			return isset($values['error']) ? $values['error'] : false;

		$valid = parent::submit($form, $source);
		if ($valid === true) {
			$this->setCouponCode($values['values'][$prefix . 'coupon_code']);
		}
		return $valid;
	}

	protected function _fromPost(\WP_Post $post)
	{
		$this->setCouponCode(\get_post_meta($post->ID, 'wre_event_coupon_code', true));
		return $this;
	}

	protected function _save($id)
	{
		\update_post_meta($id, 'wre_event_coupon_code', $this->getCouponCode());
		return $this;
	}

	function getEventPriority()
	{
		return isset($this->eventPriority) ? \intval($this->eventPriority) : 50;
	}

	public function setEventPriority($priority)
	{
		$this->eventPriority = \intval($priority);
		return $this;
	}

	protected function _install()
	{
		\add_filter('lws_woorewards_wc_order_done_' . $this->getPoolName(), array($this, 'orderDone'), $this->getEventPriority());
		\add_filter('woocommerce_cart_totals_coupon_label', array($this, 'changeCouponLabel'), 10, 2);
	}

	function getCouponCode()
	{
		return isset($this->couponCode) ? $this->couponCode : '';
	}

	public function setCouponCode($code)
	{
		// Add code and regular expression verification
		$this->couponCode = $code;
		return $this;
	}

	/** Manage backward compatibility if admin didn't update WooRewards
	 *	to at least 4.9 but updated this addon anyway. */
	private function getMyPointsAmount($user, $cartOrOrder)
	{
		if (\method_exists($this, 'getGain')) {
			return $this->getGain(array(
				'user'    => $user,
				'order'   => $cartOrOrder,
				'default' => 0,
			), true);
		} else {
			return $this->getMultiplier();
		}
	}

	function changeCouponLabel($label, $coupon)
	{
		if ($coupon && $this->isCouponValid($coupon->get_code())) {
			$points = $this->getMyPointsAmount(
				\LWS\Adminpanel\Tools\Conveniences::getCustomer(\wp_get_current_user(), \WC()->cart),
				\WC()->cart
			);
			$text = \LWS_WooRewards::formatPoints($points, $this->getPoolName());
			if ($pool = $this->getPool()) {
				$format = $pool->getOption('point_format');
				$unit   = $this->getPoolPointUnit($points);
				if($unit && $format)
					$text = sprintf($format, $points, $unit);
			}
			$text = \apply_filters('lws_woorewards_point_with_unit_format', $text, $points, $this->getPoolName(), $pool);
			$label = \wp_kses(sprintf(__("Reward : %s", 'woorewards-points-for-coupon'), $text), array());
		}
		return $label;
	}

	protected function getPoolPointUnit($count=0)
	{
		$value = '';
		if ($pool = $this->getPool()) {
			$value = $pool->getOption('disp_point_name_'.($count==1 ? 'singular' : 'plural'));
			if (!$value && $count != 1)
				$value = $pool->getOption('disp_point_name_singular');
		}
		if (!$value)
			$value = ($count != 1 ? __("Points", 'woorewards-points-for-coupon') : __("Point", 'woorewards-points-for-coupon'));
		return $value;
	}

	function isCouponValid($code)
	{
		$pattern = $this->getCouponCode();
		return \apply_filters(
			'lws_woorewards_event_use_coupon_is_valid',
			\fnmatch(\strtolower($pattern), \strtolower($code)),
			$code, $pattern, $this
		);
	}

	function orderDone($order)
	{
		if (!$this->acceptOrder($order->order))
			return $order;
		if (!$this->isValidCurrency($order->order))
			return $order;

		$userId = \LWS\Adminpanel\Tools\Conveniences::getCustomerId(false, $order->order);
		if (!$userId) {
			return $order;
		}
		$info = $userId;
		if (defined('LWS_WOOREWARDS_PRO_VERSION') && \version_compare(LWS_WOOREWARDS_PRO_VERSION, '4.9.0', '>=')) {
			$info = array(
				'user'  => $userId,
				'order' => $order->order,
			);
		}

		foreach ($order->order->get_coupon_codes() as $code) {
			if ($this->isCouponValid($code)) {
				if ($points = \apply_filters('trigger_' . $this->getType(), 1, $this, $order->order)) {
					$reason = \LWS\WOOREWARDS\Core\Trace::byOrder($order->order)
						->setProvider($order->order->get_customer_id('edit'))
						->setReason(array("Used coupon %s on order #%s", $code, $order->order->get_order_number()), 'woorewards-points-for-coupon');

					$this->addPoint($info, $reason, $points);
				}
			}
		}
		return $order;
	}

	function getPointsForProduct(\WC_Product $product)
	{
		return 0;
	}

	function getPointsForCart(\WC_Cart $cart)
	{
		if (!$this->acceptOrigin('checkout'))
		return 0;
		if (!$this->acceptCart($cart))
			return 0;
		if (!$this->isValidCurrency())
			return 0;
		$points = 0;
		foreach (\WC()->cart->get_applied_coupons() as $code) {
			if ($this->isCouponValid($code)) {
				$points += $this->getMyPointsAmount(
					\LWS\Adminpanel\Tools\Conveniences::getCustomer(\wp_get_current_user(), $cart),
					$cart
				);
			}
		}
		return $points;
	}

	function getPointsForOrder(\WC_Order $order)
	{
		if (!$this->acceptOrder($order))
			return 0;
		if (!$this->isValidOriginByOrder($order))
			return 0;
		if (!$this->isValidCurrency($order))
			return 0;
		$points = 0;
		foreach ($order->get_coupon_codes() as $code) {
			if ($this->isCouponValid($code)) {
				$points += $this->getMyPointsAmount(
					\LWS\Adminpanel\Tools\Conveniences::getCustomer(false, $order),
					$order
				);
			}
		}
		return $points;
	}

	/** Never call, only to have poedit/wpml able to extract the sentance. */
	private function poeditDeclare()
	{
		__("Used coupon %s on order #%s", 'woorewards-points-for-coupon');
	}
}
